package com.link.core.util;

import com.link.core.objlocalhandle.kafka.KafkaEvent;
import com.link.core.objlocalhandle.kafka.KafkaEventFactory;
import com.link.core.objlocalhandle.kafka.KafkaEventHandler;
import com.link.core.objlocalhandle.kafka.KafkaEventProducer;
import com.lmax.disruptor.EventHandler;
import com.lmax.disruptor.RingBuffer;
import com.lmax.disruptor.dsl.Disruptor;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.support.TransactionSynchronizationAdapter;
import org.springframework.transaction.support.TransactionSynchronizationManager;
import org.springframework.util.CollectionUtils;

public class ObjLocalKafkaUtils {
   private static Logger logger = LoggerFactory.getLogger(ObjLocalKafkaUtils.class);
   private static ThreadLocal<List<KafkaEvent>> KAFKA_EVENTS = new ThreadLocal();
   private static int INIT_SIZE = 2048;
   private static Disruptor<KafkaEvent> disruptor;

   private static Disruptor<KafkaEvent> getInstance() {
      if (null == disruptor) {
         Class var0 = ObjLocalKafkaUtils.class;
         synchronized(ObjLocalKafkaUtils.class) {
            if (null == disruptor) {
               KafkaEventFactory kafkaEventFactory = new KafkaEventFactory();
               Executor executor = Executors.newCachedThreadPool();
               disruptor = new Disruptor(kafkaEventFactory, INIT_SIZE, executor);
               disruptor.handleEventsWith(new EventHandler[]{new KafkaEventHandler()});
               disruptor.start();
            }
         }
      }

      return disruptor;
   }

   public static void publish(KafkaEvent kafkaEvent) {
      logger.info("向消息队列扔消息");
      boolean actualTransactionActive = TransactionSynchronizationManager.isActualTransactionActive();
      if (!actualTransactionActive) {
         RingBuffer<KafkaEvent> ringBuffer = disruptor.getRingBuffer();
         KafkaEventProducer producer = new KafkaEventProducer(ringBuffer);
         producer.onData(kafkaEvent);
      } else {
         List<KafkaEvent> kafkaEvents = (List)KAFKA_EVENTS.get();
         if (CollectionUtils.isEmpty((Collection)kafkaEvents)) {
            kafkaEvents = new ArrayList();
            KAFKA_EVENTS.set(kafkaEvents);
            TransactionSynchronizationManager.registerSynchronization(new TransactionSynchronizationAdapter() {
               public void afterCommit() {
                  super.afterCommit();
                  List<KafkaEvent> kafkaEvents = (List)ObjLocalKafkaUtils.KAFKA_EVENTS.get();
                  RingBuffer<KafkaEvent> ringBuffer = ObjLocalKafkaUtils.disruptor.getRingBuffer();
                  KafkaEventProducer producer = new KafkaEventProducer(ringBuffer);
                  Iterator var4 = kafkaEvents.iterator();

                  while(var4.hasNext()) {
                     KafkaEvent temp = (KafkaEvent)var4.next();
                     producer.onData(temp);
                  }

               }

               public void afterCompletion(int status) {
                  super.afterCompletion(status);
                  ObjLocalKafkaUtils.KAFKA_EVENTS.remove();
               }
            });
         }

         ((List)kafkaEvents).add(kafkaEvent);
      }

   }

   static {
      KafkaEventFactory kafkaEventFactory = new KafkaEventFactory();
      Executor executor = Executors.newCachedThreadPool();
      disruptor = new Disruptor(kafkaEventFactory, INIT_SIZE, executor);
      disruptor.handleEventsWith(new EventHandler[]{new KafkaEventHandler()});
      disruptor.start();
   }
}
